from typing import Union

from yarl import URL

from ..enums import RequestTypes
from ..http import BaseHTTP


class Images(BaseHTTP):
    """Images Api endpoints documented in https://bluedocs.page/fluxpoint-api/images"""

    def __str__(self) -> str:
        return '<Images>'

    async def neko(self) -> Union[URL, str]:
        """
        Returns a random image of nekos

        :return: Url of the image
        :rtype: Union[:class:`yarl.URL` , :class:`str`]
        """
        return (await self.request(RequestTypes.GET, 'api/sfw/img/neko', _base_url='https://gallery.fluxpoint.dev/'))['file']

    async def maid(self) -> Union[URL, str]:
        """
        Returns a random image of maids

        :return: Url of the image
        :rtype: Union[:class:`yarl.URL` , :class:`str`]
        """
        return (await self.request(RequestTypes.GET, '/api/sfw/img/maid', _base_url='https://gallery.fluxpoint.dev/'))['file']

    async def nekopara(self) -> Union[URL, str]:
        """
        Returns a random image from nekopara

        :return: Url of the image
        :rtype: Union[:class:`yarl.URL` , :class:`str`]
        """
        return (await self.request(RequestTypes.GET, '/api/sfw/img/nekopara', _base_url='https://gallery.fluxpoint.dev/'))['file']

    async def azurlane(self) -> Union[URL, str]:
        """
        Returns a random image from azurlane

        :return: Url of the image
        :rtype: Union[:class:`yarl.URL` , :class:`str`]
        """
        return (await self.request(RequestTypes.GET, '/api/sfw/img/azurlane', _base_url='https://gallery.fluxpoint.dev/'))['file']

    async def senko(self) -> Union[URL, str]:
        """
        Returns a random image from senko

        :return: Url of the image
        :rtype: Union[:class:`yarl.URL` , :class:`str`]
        """
        return (await self.request(RequestTypes.GET, '/api/sfw/img/senko', _base_url='https://gallery.fluxpoint.dev/'))['file']

    async def ddlc(self) -> Union[URL, str]:
        """
        Returns a random image from Doki Doki Literature Club

        :return: Url of the image
        :rtype: Union[:class:`yarl.URL` , :class:`str`]
        """
        return (await self.request(RequestTypes.GET, '/api/sfw/img/ddlc', _base_url='https://gallery.fluxpoint.dev/'))['file']

    async def wallpaper(self) -> Union[URL, str]:
        """
        Returns a random wallpaper image

        :return: Url of the image
        :rtype: Union[:class:`yarl.URL` , :class:`str`]
        """
        return (await self.request(RequestTypes.GET, '/api/sfw/img/wallpaper', _base_url='https://gallery.fluxpoint.dev/'))['file']

    async def anime(self) -> Union[URL, str]:
        """
        Returns a random anime image

        :return: Url of the image
        :rtype: Union[:class:`yarl.URL` , :class:`str`]
        """
        return (await self.request(RequestTypes.GET, '/api/sfw/img/anime', _base_url='https://gallery.fluxpoint.dev/'))['file']

    async def meme(self) -> Union[URL, str]:
        """
        Returns a random meme

        :return: Url of the image
        :rtype: Union[:class:`yarl.URL` , :class:`str`]
        """
        return (await self.request(RequestTypes.GET, '/api/sfw/img/meme', _base_url='https://gallery.fluxpoint.dev/'))['file']

    async def nou(self) -> Union[URL, str]:
        """
        Returns a random No U image

        :return: Url of the image
        :rtype: Union[:class:`yarl.URL` , :class:`str`]
        """
        return (await self.request(RequestTypes.GET, '/api/sfw/img/nou', _base_url='https://gallery.fluxpoint.dev/'))['file']

    async def pog(self) -> Union[URL, str]:
        """
        Returns a random pog meme image

        :return: Url of the image
        :rtype: Union[:class:`yarl.URL` , :class:`str`]
        """
        return (await self.request(RequestTypes.GET, '/api/sfw/img/pog', _base_url='https://gallery.fluxpoint.dev/'))['file']

    async def cat(self) -> Union[URL, str]:
        """
        Returns a random cat image

        :return: Url of the image
        :rtype: Union[:class:`yarl.URL` , :class:`str`]
        """
        return (await self.request(RequestTypes.GET, '/api/sfw/img/cat', _base_url='https://gallery.fluxpoint.dev/'))['file']

    async def dog(self) -> Union[URL, str]:
        """
        Returns a random dog image

        :return: Url of the image
        :rtype: Union[:class:`yarl.URL` , :class:`str`]
        """
        return (await self.request(RequestTypes.GET, '/api/sfw/img/dog', _base_url='https://gallery.fluxpoint.dev/'))['file']

    async def lizard(self) -> Union[URL, str]:
        """
        Returns a random lizard image

        :return: Url of the image
        :rtype: Union[:class:`yarl.URL` , :class:`str`]
        """
        return (await self.request(RequestTypes.GET, '/api/sfw/img/lizard', _base_url='https://gallery.fluxpoint.dev/'))['file']



__title__ = 'fluxpoint'
__author__ = 'Dhruva Shaw'
__license__ = 'GNU GENERAL PUBLIC LICENSE'
__copyright__ = 'Copyright 2021-present Dhruvacube'
__version__ = '0.1.1'
__path__ = __import__('pkgutil').extend_path(__path__, __name__)

from typing import NamedTuple, Literal
import logging

from .http import *
from .enums import *
from .paths import *
from .client import *
from .errors import *


class VersionInfo(NamedTuple):
    major: int
    minor: int
    micro: int
    releaselevel: Literal["alpha", "beta", "candidate", "final"]
    serial: int


version_info: VersionInfo = VersionInfo(
    major=0, minor=1, micro=1, releaselevel='final', serial=0)

logging.getLogger(__name__).addHandler(logging.NullHandler())

del logging, NamedTuple, Literal, VersionInfo



from typing import Literal, Optional, Dict, Union, List
import io
from ..http import BaseHTTP
from ..enums import RequestTypes


class Square:
    """Square Image class

        :param width: Set the width of the shape
        :type width: int
        :param height: Set the height of the shape
        :type height: int
        :param color: Set the shape to be a color from name/hex/rgb/rgba use rgba for transparency, defaults to None
        :type color: Optional[str], optional
        :param round:  Make the borders of the shape round. (Default 0), defaults to None
        :type round: Optional[int], optional
        :param x: Position the image in pixels, defaults to None
        :type x: Optional[int], optional
        :param y: Position the image in pixels, defaults to None
        :type y: Optional[int], optional
    """
    __slots__ = ["width", "height", "color", "round", "x", "y"]

    def __init__(
        self,
        width: int,
        height: int,
        color: Optional[str] = None,
        round: Optional[int] = None,
        x: Optional[int] = None,
        y: Optional[int] = None
    ) -> None:
        if color is None:
            self.color = color
        self.width = width
        self.height = height
        if round is not None:
            self.round = round
        if x is not None:
            self.x = x
        if y is not None:
            self.y = y

    def to_dict(self) -> dict:
        """Converts the class to a dictionary"""
        return_dict = {'type': 'bitmap'}
        for i in self.__slots__:
            if hasattr(self, i):
                return_dict[i] = getattr(self, i)
        return return_dict


class Triangle:
    """Triangle Image class

        :param color: Set the shape to be a color from name/hex/rgb/rgba use rgba for transparency
        :type color: Optional[str]
        :param width: Set the width of the shape
        :type width: int
        :param height: Set the height of the shape
        :type height: int
        :param cut: Choose where the missing peice of the triangle is
        :type cut: Literal[topleft, topright, bottomleft, bottomright]
        :param x: Position the image in pixels, defaults to None
        :type x: Optional[int], optional
        :param y: Position the image in pixels, defaults to None
        :type y: Optional[int], optional
    """

    __slots__ = ["width", "height", "cut", "color", "x", "y"]

    def __init__(
        self,
        width: int,
        height: int,
        cut: Literal['topleft', 'topright', 'bottomleft', 'bottomright'],
        color: Optional[str] = None,
        x: Optional[int] = None,
        y: Optional[int] = None
    ) -> None:
        if color is not None:
            self.color = color
        self.width = width
        self.height = height
        self.cut = cut
        if x is not None:
            self.x = x
        if y is not None:
            self.y = y

    def to_dict(self) -> dict:
        """Converts the class to a dictionary"""
        return_dict = {'type': 'triangle'}
        for i in self.__slots__:
            if hasattr(self, i):
                return_dict[i] = getattr(self, i)
        return return_dict


class Circle:
    """Circle Image class

        :param color: Set the shape to be a color from name/hex/rgb/rgba use rgba for transparency
        :type color: Optional[str]
        :param radius: Set the size of the circle from the center point
        :type radius: Optional[int]
        :param x: Position the image in pixels, defaults to None
        :type x: Optional[int], optional
        :param y: Position the image in pixels, defaults to None
        :type y: Optional[int], optional
    """
    __slots__ = ["color", "radius", "x", "y"]

    def __init__(self, color: Optional[str] = None, radius: Optional[int] = None, x: Optional[int] = None, y: Optional[int] = None) -> None:
        if color is not None:
            self.color = color
        if radius is not None:
            self.radius = radius
        if x is not None:
            self.x = x
        if y is not None:
            self.y = y

    def to_dict(self) -> dict:
        """Converts the class to a dictionary"""
        given_dict = {'type': 'circle'}
        for i in self.__slots__:
            if hasattr(self, i):
                given_dict[i] = getattr(self, i)
        return given_dict


class ImageUrl:
    """ImageUrl Image class

        :param url: https://website.com/image.png
        :type url: str
        :param cache: Cache the image server-side so it can be easily loaded again such as background images. (Not Recommended for Avatars), defaults to False
        :type cache: Optional[bool], optional
        :param width: Set the width of the image, defaults to None
        :type width: Optional[int], optional
        :param height: Set the height of the image, defaults to None
        :type height: Optional[int], optional
        :param maxwidth: Set the max width so the image can scale properly, defaults to None
        :type maxwidth: Optional[int], optional
        :param maxheight: Set the max height so the image can scale properly, defaults to None
        :type maxheight: Optional[int], optional
        :param round: Make the borders of the image round for stuff like circle avatars, defaults to None
        :type round: Optional[int], optional
        :param x: Position the image in pixels, defaults to None
        :type x: Optional[int], optional
        :param y: Position the image in pixels, defaults to None
        :type y: Optional[int], optional
    """

    __slots__ = ["url", "cache", "width",
                 "height", "maxwidth", "maxheight", "round"]

    def __init__(
        self,
        url: str,
        cache: Optional[bool] = False,
        width: Optional[int] = None,
        height: Optional[int] = None,
        maxwidth: Optional[int] = None,
        maxheight: Optional[int] = None,
        round: Optional[int] = None,
        x: Optional[int] = None,
        y: Optional[int] = None
    ) -> None:
        self.url = url
        self.cache = cache
        if width is not None:
            self.width = width
        if height is not None:
            self.height = height
        if maxwidth is not None:
            self.maxwidth = maxwidth
        if maxheight is not None:
            self.maxheight = maxheight
        if round is not None:
            self.round = round
        if x is not None:
            self.x = x
        if y is not None:
            self.y = y

    def to_dict(self) -> dict:
        """Converts the class to a dictionary"""
        return_dict = {"type": "url"}
        for i in self.__slots__:
            if hasattr(self, i):
                return_dict[i] = getattr(self, i)
        return return_dict


class Text:
    """Text class for custom image

        :param text: Set the text
        :type text: str

        :param size: Set the font size of the text, defaults to 16
        :type size: Optional[int], optional

        :param font: Set the font to use for the text, defaults to "Sans Serif"
        :type font: Optional[str], optional

        :param color: Set the text color from name/hex/rgb/rgba use rgba for transparency, defaults to "white"
        :type color: Optional[str], optional

        :param back: Set the background color for the text box from name/hex/rgb/rgba use rgba for transparency, defaults to None
        :type back: Optional[str], optional

        :param x: Position the text in pixels, defaults to 0
        :type x: Optional[int], optional

        :param y: Position the text in pixels, defaults to 0
        :type y: Optional[int], optional

        :param align: Align the text from left/center/right (Default left), defaults to "l"
        :type align: Optional[Literal[l,m,r]], optional

        :param bold: Set the text to bold style, defaults to False
        :type bold: Optional[bool], optional

        :param italics: Set the text to italics style, defaults to False
        :type italics: Optional[bool], optional

        :param underline: Set the text to underline style, defaults to False
        :type underline: Optional[bool], optional

        :param line: Set the text to underline style, defaults to 1.0
        :type line: Optional[float], optional

        :param weight: Increase the boldness of the tex, defaults to 500
        :type weight: Optional[int], optional

        :param width: Set the max width of the text box before it wraps to a newline, defaults to None
        :type width: Optional[int], optional

        :param height: et the max height of the text box to clip extra text, defaults to None
        :type height: Optional[int], optional

        :param outline: Show text outline with default options, defaults to False
        :type outline: Optional[bool], optional

        :param outlinewidth: Set the outline size from the text, defaults to 9
        :type outlinewidth: Optional[int], optional

        :param outlinecolor: Set the outline color from name/hex/rgb/rgba use rgba for transparency, defaults to "black"
        :type outlinecolor: Optional[str], optional

        :param outlineblur: Make the outline transparency less visible for a cool effect, defaults to 1
        :type outlineblur: Optional[int], optional
    """
    __slots__ = [
        "text",
        "size",
        "font",
        "color",
        "back",
        "x",
        "y",
        "align",
        "bold",
        "italics",
        "underline",
        "line",
        "weight",
        "width",
        "height",
        "outline",
        "outlinewidth",
        "outlinecolor",
        "outlineblur"
    ]

    def __init__(
        self,
        text: str,
        size: Optional[int] = 16,
        font: Optional[str] = "Sans Serif",
        color: Optional[str] = "white",
        back: Optional[str] = None,
        x: Optional[int] = 0,
        y: Optional[int] = 0,
        align: Optional[Literal["l", "m", "r"]] = "l",
        bold: Optional[bool] = False,
        italics: Optional[bool] = False,
        underline: Optional[bool] = False,
        line: Optional[float] = 1.0,
        weight: Optional[int] = 500,
        width: Optional[int] = None,
        height: Optional[int] = None,
        outline: Optional[bool] = False,
        outlinewidth: Optional[int] = 9,
        outlinecolor: Optional[str] = "black",
        outlineblur: Optional[int] = 1,
    ) -> None:
        self.text = text
        self.size = size
        self.font = font
        self.color = color
        if back is not None:
            self.back = back
        self.x = x
        self.y = y
        self.align = align
        self.bold = bold
        self.italics = italics
        self.underline = underline
        self.line = line
        self.weight = weight
        if width is not None:
            self.width = width
        if height is not None:
            self.height = height
        self.outline = outline
        self.outlinewidth = outlinewidth
        self.outlinecolor = outlinecolor
        self.outlineblur = outlineblur

    def to_dict(self) -> dict:
        """Converts the class to a dictionary"""
        return_dict = {}
        for i in self.__slots__:
            if hasattr(self, i):
                return_dict[i] = getattr(self, i)
        return return_dict


class ImageGenerator(BaseHTTP):
    """Custome image generator Api endpoints documented in https://bluedocs.page/fluxpoint-api/imagegen"""

    def __str__(self):
        return "<Image Generator>"

    async def test(self) -> Union[Dict, io.IOBase]:
        """Test the image generator

        :returns: The response from the server
        :rtype: Union[Dict, io.IOBase]
        """
        return await self.request(RequestTypes.GET, '/test/image', return_bytes=True, return_json=False)  # skipcq: TYP-005

    async def customimage(
        self,
        type: Literal["bitmap", "image"],
        width: int,
        height: int,
        color: str,
        images: Optional[List[Union[ImageUrl,
                                    Square, Triangle, Circle]]] = None,
        texts: Optional[List[Text]] = None
    ) -> Union[Dict, io.IOBase]:
        """Get the created image gen image.

        :param type: Image type
        :type type: Literal[bitmap, image]
        :param width: Overall width of the image
        :type width: int
        :param height: Overall height of the image
        :type height: int
        :param color: Background colour of the image
        :type color: str
        :param images: The image(s) that you want to embed in image, defaults to []
        :type images: Optional[List[Union[ImageUrl, Square, Triangle, Circle]]], optional
        :param texts: The text(s) that you want to embed in image, defaults to []
        :type texts: Optional[List[Text]], optional

        :return: The custom generated image bytes data
        :rtype: Union[Dict, io.IOBase]
        """
        if images is None:
            images = []
        if texts is None:
            texts = []
        json_data = {
            "Base": {
                "type": type,
                "width": width,
                "height": height,
                "color": color
            },
            "Images": list(map(lambda x: x.to_dict(), images)),
            "Texts": list(map(lambda x: x.to_dict(), texts))
        }
        # skipcq : TYP-005
        return await self.request(RequestTypes.POST, '/gen/custom', return_bytes=True, return_json=False, json=json_data)



class HttpException(Exception):
    """
    Fluxpoint base exception class use this base class to catch any Fluxpoint errors.
    """

    def __init__(self, status: int, message: str) -> None:
        self.status: int = status
        self.message: str = message
        super().__init__(message)


class ApiError(HttpException):
    """
    Raised when Fluxpoint has an error it does not know how to handle
    """

    def __init__(self, message: str) -> None:
        super().__init__(500, message)


class WrongReturnType(Exception):
    """
    Exception raised when the when wrong return type is given
    """


class Unauthorised(HttpException):
    """
    Raised for an API 401
    """

    def __init__(self, message: str) -> None:
        super().__init__(401, message)


class RateLimited(HttpException):
    """
    You are exceeding the API's rate limits and built in Ratelimit handler
    Essentially a 429
    """

    def __init__(self, message: str) -> None:
        super().__init__(429, message)


class ParameterError(HttpException):
    """
    Parameters passed were not Sufficient
    """

    def __init__(self, message: str) -> None:
        super().__init__(400, message)



from typing import Optional, Union

from yarl import URL

from ..enums import RequestTypes
from ..http import BaseHTTP


class NSFW(BaseHTTP):
    """NSFW Api endpoints documented in https://bluedocs.page/fluxpoint-api/nsfw"""

    def __str__(self) -> str:
        return '<Gifs>'

    async def azurlane_nsfw(self) -> Union[URL, str]:
        """
        Returns a random nsfw azurlane image

        :return: Url of the image
        :rtype: Union[:class:`yarl.URL` , :class:`str`]
        """
        return (await self.request(RequestTypes.GET, 'api/nsfw/img/azurlane', _base_url='https://gallery.fluxpoint.dev/'))['file']

    async def feet_nsfw(self, gif: Optional[bool] = False) -> Union[URL, str]:
        """
        Returns a random nsfw feet image/gif

        :param gif: If image returned should be gif image, defaults to False
        :type gif: Optional[bool], optional
        :return: Url of the image
        :rtype: Union[:class:`yarl.URL` , :class:`str`]
        """
        return (
            await self.request(
                RequestTypes.GET,
                '/api/nsfw/img/feet' if not gif else '/api/nsfw/gif/feet',
                _base_url='https://gallery.fluxpoint.dev/'
            )
        )['file']

    async def cum_nsfw(self, gif: Optional[bool] = False) -> Union[URL, str]:
        """
        Returns a random nsfw blowjob image/gif

        :param gif: If image returned should be gif image, defaults to False
        :type gif: Optional[bool], optional
        :return: Url of the image
        :rtype: Union[:class:`yarl.URL` , :class:`str`]
        """
        return (
            await self.request(
                RequestTypes.GET,
                '/api/nsfw/img/cum' if not gif else '/api/nsfw/gif/cum',
                _base_url='https://gallery.fluxpoint.dev/'
            )
        )['file']

    async def blowjob_nsfw(self, gif: Optional[bool] = False) -> Union[URL, str]:
        """
        Returns a random nsfw blowjob image/gif

        :param gif: If image returned should be gif image, defaults to False
        :type gif: Optional[bool], optional
        :return: Url of the image
        :rtype: Union[:class:`yarl.URL` , :class:`str`]
        """
        return (
            await self.request(
                RequestTypes.GET,
                '/api/nsfw/img/blowjob' if not gif else '/api/nsfw/gif/blowjob',
                _base_url='https://gallery.fluxpoint.dev/'
            )
        )['file']

    async def solo_girl_nsfw(self, gif: Optional[bool] = False) -> Union[URL, str]:
        """
        Returns a random nsfw solo girl image/gif

        :param gif: If image returned should be gif image, defaults to False
        :type gif: Optional[bool], optional
        :return: Url of the image
        :rtype: Union[:class:`yarl.URL` , :class:`str`]
        """
        return (
            await self.request(
                RequestTypes.GET,
                '/api/nsfw/img/solo' if not gif else '/api/nsfw/gif/solo',
                _base_url='https://gallery.fluxpoint.dev/'
            )
        )['file']

    async def neko_nsfw(self, gif: Optional[bool] = False) -> Union[URL, str]:
        """
        Returns a random nsfw neko image/gif

        :param gif: If image returned should be gif image, defaults to False
        :type gif: Optional[bool], optional
        :return: Url of the image
        :rtype: Union[:class:`yarl.URL` , :class:`str`]
        """
        return (
            await self.request(
                RequestTypes.GET,
                '/api/nsfw/img/neko' if not gif else '/api/nsfw/gif/neko',
                _base_url='https://gallery.fluxpoint.dev/'
            )
        )['file']

    async def boobs_nsfw(self, gif: Optional[bool] = False) -> Union[URL, str]:
        """
        Returns a random nsfw boobs image/gif

        :param gif: If image returned should be gif image, defaults to False
        :type gif: Optional[bool], optional
        :return: Url of the image
        :rtype: Union[:class:`yarl.URL` , :class:`str`]
        """
        return (
            await self.request(
                RequestTypes.GET,
                '/api/nsfw/img/boobs' if not gif else '/api/nsfw/gif/boobs',
                _base_url='https://gallery.fluxpoint.dev/'
            )
        )['file']

    async def anal_nsfw(self, gif: Optional[bool] = False) -> Union[URL, str]:
        """
        Returns a random nsfw anal image/gif

        :param gif: If image returned should be gif image, defaults to False
        :type gif: Optional[bool], optional
        :return: Url of the image
        :rtype: Union[:class:`yarl.URL` , :class:`str`]
        """
        return (
            await self.request(
                RequestTypes.GET,
                '/api/nsfw/img/anal' if not gif else '/api/nsfw/gif/anal',
                _base_url='https://gallery.fluxpoint.dev/'
            )
        )['file']

    async def pussy_nsfw(self, gif: Optional[bool] = False) -> Union[URL, str]:
        """
        Returns a random nsfw pussy image/gif

        :param gif: If image returned should be gif image, defaults to False
        :type gif: Optional[bool], optional
        :return: Url of the image
        :rtype: Union[:class:`yarl.URL` , :class:`str`]
        """
        return (
            await self.request(
                RequestTypes.GET,
                '/api/nsfw/img/pussy' if not gif else '/api/nsfw/gif/pussy',
                _base_url='https://gallery.fluxpoint.dev/'
            )
        )['file']

    async def yuri_nsfw(self, gif: Optional[bool] = False) -> Union[URL, str]:
        """
        Returns a random nsfw yuri image/gif

        :param gif: If image returned should be gif image, defaults to False
        :type gif: Optional[bool], optional
        :return: Url of the image
        :rtype: Union[:class:`yarl.URL` , :class:`str`]
        """
        return (
            await self.request(
                RequestTypes.GET,
                '/api/nsfw/img/yuri' if not gif else '/api/nsfw/gif/yuri',
                _base_url='https://gallery.fluxpoint.dev/'
            )
        )['file']

    async def bdsm_nsfw(self, gif: Optional[bool] = False) -> Union[URL, str]:
        """
        Returns a random nsfw bdsm image/gif

        :param gif: If image returned should be gif image, defaults to False
        :type gif: Optional[bool], optional
        :return: Url of the image
        :rtype: Union[:class:`yarl.URL` , :class:`str`]
        """
        return (
            await self.request(
                RequestTypes.GET,
                '/api/nsfw/img/yuri' if not gif else '/api/nsfw/gif/yuri',
                _base_url='https://gallery.fluxpoint.dev/'
            )
        )['file']

    async def futa_nsfw(self, gif: Optional[bool] = False) -> Union[URL, str]:
        """
        Returns a random nsfw futa image/gif

        :param gif: If image returned should be gif image, defaults to False
        :type gif: Optional[bool], optional
        :return: Url of the image
        :rtype: Union[:class:`yarl.URL` , :class:`str`]
        """
        return (
            await self.request(
                RequestTypes.GET,
                '/api/nsfw/img/futa' if not gif else '/api/nsfw/gif/futa',
                _base_url='https://gallery.fluxpoint.dev/'
            )
        )['file']

    async def hentai_nsfw(self) -> Union[URL, str]:
        """
        Returns a random nsfw hentai gif

        :return: Url of the image
        :rtype: Union[:class:`yarl.URL` , :class:`str`]
        """
        return (await self.request(RequestTypes.GET, 'api/nsfw/gif/hentai', _base_url='https://gallery.fluxpoint.dev/'))['file']

    async def spank_nsfw(self) -> Union[URL, str]:
        """
        Returns a random nsfw spank gif

        :return: Url of the image
        :rtype: Union[:class:`yarl.URL` , :class:`str`]
        """
        return (await self.request(RequestTypes.GET, 'api/nsfw/gif/spank', _base_url='https://gallery.fluxpoint.dev/'))['file']

    async def ass_nsfw(self, gif: Optional[bool] = False) -> Union[URL, str]:
        """
        Returns a random nsfw ass image/gif

        :param gif: If image returned should be gif image, defaults to False
        :type gif: Optional[bool], optional
        :return: Url of the image
        :rtype: Union[:class:`yarl.URL` , :class:`str`]
        """
        return (
            await self.request(
                RequestTypes.GET,
                '/api/nsfw/img/ass' if not gif else '/api/nsfw/gif/ass',
                _base_url='https://gallery.fluxpoint.dev/'
            )
        )['file']

    async def kitsune_nsfw(self, gif: Optional[bool] = False) -> Union[URL, str]:
        """
        Returns a random nsfw kitsune image/gif

        :param gif: If image returned should be gif image, defaults to False
        :type gif: Optional[bool], optional
        :return: Url of the image
        :rtype: Union[:class:`yarl.URL` , :class:`str`]
        """
        return (
            await self.request(
                RequestTypes.GET,
                '/api/nsfw/img/kitsune' if not gif else '/api/nsfw/gif/kitsune',
                _base_url='https://gallery.fluxpoint.dev/'
            )
        )['file']

    async def femdom_nsfw(self, gif: Optional[bool] = False) -> Union[URL, str]:
        """
        Returns a random nsfw femdom image/gif

        :param gif: If image returned should be gif image, defaults to False
        :type gif: Optional[bool], optional
        :return: Url of the image
        :rtype: Union[:class:`yarl.URL` , :class:`str`]
        """
        return (
            await self.request(
                RequestTypes.GET,
                '/api/nsfw/img/femdom' if not gif else '/api/nsfw/gif/femdom',
                _base_url='https://gallery.fluxpoint.dev/'
            )
        )['file']

    async def nekopara_nsfw(self) -> Union[URL, str]:
        """
        Returns a random nsfw nekopara image

        :return: Url of the image
        :rtype: Union[:class:`yarl.URL` , :class:`str`]
        """
        return (await self.request(RequestTypes.GET, 'api/nsfw/img/nekopara', _base_url='https://gallery.fluxpoint.dev/'))['file']

    async def lewd_nsfw(self) -> Union[URL, str]:
        """
        Returns a random nsfw lewd image

        :return: Url of the image
        :rtype: Union[:class:`yarl.URL` , :class:`str`]
        """
        return (await self.request(RequestTypes.GET, 'api/nsfw/img/lewd', _base_url='https://gallery.fluxpoint.dev/'))['file']

    async def pantyhose_nsfw(self) -> Union[URL, str]:
        """
        Returns a random nsfw pantyhose image

        :return: Url of the image
        :rtype: Union[:class:`yarl.URL` , :class:`str`]
        """
        return (await self.request(RequestTypes.GET, 'api/nsfw/img/pantyhose', _base_url='https://gallery.fluxpoint.dev/'))['file']

    async def cosplay_nsfw(self) -> Union[URL, str]:
        """
        Returns a random nsfw cosplay image

        :return: Url of the image
        :rtype: Union[:class:`yarl.URL` , :class:`str`]
        """
        return (await self.request(RequestTypes.GET, 'api/nsfw/img/cosplay', _base_url='https://gallery.fluxpoint.dev/'))['file']

    async def petplay_nsfw(self) -> Union[URL, str]:
        """
        Returns a random nsfw petplay image

        :return: Url of the image
        :rtype: Union[:class:`yarl.URL` , :class:`str`]
        """
        return (await self.request(RequestTypes.GET, 'api/nsfw/img/petplay', _base_url='https://gallery.fluxpoint.dev/'))['file']

    async def gasm_nsfw(self) -> Union[URL, str]:
        """
        Returns a random nsfw ahego image

        :return: Url of the image
        :rtype: Union[:class:`yarl.URL` , :class:`str`]
        """
        return (await self.request(RequestTypes.GET, 'api/nsfw/img/gasm', _base_url='https://gallery.fluxpoint.dev/'))['file']

    async def ahego_nsfw(self) -> Union[URL, str]:
        """
        Returns a random nsfw ahego image

        :return: Url of the image
        :rtype: Union[:class:`yarl.URL` , :class:`str`]
        """
        return (await self.request(RequestTypes.GET, 'api/nsfw/img/gasm', _base_url='https://gallery.fluxpoint.dev/'))['file']

    async def trap_nsfw(self) -> Union[URL, str]:
        """
        Returns a random nsfw trap image

        :return: Url of the image
        :rtype: Union[:class:`yarl.URL` , :class:`str`]
        """
        return (await self.request(RequestTypes.GET, 'api/nsfw/img/trap', _base_url='https://gallery.fluxpoint.dev/'))['file']

    async def anus_nsfw(self) -> Union[URL, str]:
        """
        Returns a random nsfw anus image

        :return: Url of the image
        :rtype: Union[:class:`yarl.URL` , :class:`str`]
        """
        return (await self.request(RequestTypes.GET, 'api/nsfw/img/anus', _base_url='https://gallery.fluxpoint.dev/'))['file']

    async def holo_nsfw(self) -> Union[URL, str]:
        """
        Returns a random nsfw holo image

        :return: Url of the image
        :rtype: Union[:class:`yarl.URL` , :class:`str`]
        """
        return (await self.request(RequestTypes.GET, 'api/nsfw/img/holo', _base_url='https://gallery.fluxpoint.dev/'))['file']

    async def yaoi_nsfw(self) -> Union[URL, str]:
        """
        Returns a random nsfw yaoi image

        :return: Url of the image
        :rtype: Union[:class:`yarl.URL` , :class:`str`]
        """
        return (await self.request(RequestTypes.GET, 'api/nsfw/img/yaoi', _base_url='https://gallery.fluxpoint.dev/'))['file']



from typing import List, Optional, Union, Dict, Tuple
import io

from yarl import URL

from ..enums import RequestTypes
from ..http import BaseHTTP


class InvalidFeature(Exception):
    """
    The feature chosen is not valid
    """


class WelcomeConfig:
    """Class to configure the welcome image parameters

        :param username: Discord username
        :type username: str
        :param avatar: avatar url
        :type avatar: Union[URL, str]
        :param background: Backgroud color code
        :type background: str
        :param members: Members no tet, defaults to None
        :type members: Optional[str], optional
        :param icon: Icon to use, defaults to None
        :type icon: Optional[str], optional
        :param banner: Banner to be put at background, defaults to None
        :type banner: Optional[str], optional
        :param color_welcome: Colour of the welcome text, defaults to None
        :type color_welcome: Optional[str], optional
        :param color_username: Colour of the username text, defaults to None
        :type color_username: Optional[str], optional
        :param color_members: Colour of the member text, defaults to None
        :type color_members: Optional[str], optional

        .. note::
            The ``color`` parameters are optional, if you don't want to use a color, just pass ``None``

        .. warning::
            The ``banner`` and ``icon`` parameters evauation would be done when :func:`Welcome.welcome` is called.
        """

    def __init__(
        self,
        username: str,
        avatar: Union[URL, str],
        background: str,
        members: Optional[str] = None,
        icon: Optional[str] = None,
        banner: Optional[str] = None,
        color_welcome: Optional[str] = None,
        color_username: Optional[str] = None,
        color_members: Optional[str] = None,
    ) -> None:
        self.username = username
        self.avatar = avatar
        self.background = background
        self.members = members

        self.banner = banner
        self.icon = icon

        self.color_members = color_members
        self.color_username = color_username
        self.color_welcome = color_welcome

    def __str__(self) -> str:
        return f'<WelcomeConfig username={self.username} avatar={self.avatar} background={self.background}>'

    def to_dict(self) -> dict:
        """A helper function to convert the class parameters to a dictionary"""
        return_dummy_dict = {
            'username': self.username,
            'avatar': self.avatar,
            'background': self.background,
            'members': self.members,
            'banner': self.banner,
            'icon': self.icon,
            'color_members': self.color_members,
            'color_username': self.color_username,
            'color_welcome': self.color_welcome,
        }
        return_dict = {}
        for i in return_dummy_dict:
            if return_dummy_dict[i] is not None:
                return_dict[i] = return_dummy_dict[i]
        return return_dict


class Welcome(BaseHTTP):
    """Welcome Images Api endpoints documented in https://bluedocs.page/fluxpoint-api/welcome"""

    async def welcome_icons(self) -> Union[List[str], Tuple[str]]:
        """Get a list of all the welcome icons

        :return: A list of all the welcome icons
        :rtype: Union[list, tuple]
        """
        return (await self.request(RequestTypes.GET, 'list/icons')).get('list')  # skipcq: TYP-005

    async def welcome_banner(self) -> Union[List[str], Tuple[str]]:
        """Get a list of all the welcome banners

        :return: A list of all the welcome banners
        :rtype: Union[list, tuple]
        """
        return (await self.request(RequestTypes.GET, 'list/banners')).get('list')  # skipcq: TYP-005

    async def welcome(self, config: WelcomeConfig) -> Union[Dict, io.IOBase]:
        """Create a welcome image

        :raises InvalidFeature: When ``banner`` or ``icons`` parameters in ``config`` are invalid

        :param config: A WelcomeConfig object
        :type config: WelcomeConfig
        :return: The image as a byte array
        :rtype: Union[dict, io.IOBase]
        """
        if (
            config.banner is not None
            and config.banner.lower() not in list(map(lambda x: x.lower(), await self.welcome_banner()))
        ):
            raise InvalidFeature(f'Banner {config.banner} not found')
        if (
            config.icon is not None
            and config.icon.lower() not in list(map(lambda x: x.lower(), await self.welcome_icons()))
        ):
            raise InvalidFeature(f'Icon {config.icon} not found')
        # skipcq: TYP-005
        return await self.request(RequestTypes.POST, '/gen/welcome', json=config.to_dict(), return_bytes=True, return_json=False)



from typing import Optional

from ..enums import RequestTypes
from ..http import BaseHTTP


class AboutUs:
    """A class which gives info from the json data of about us

    :param json_data: The json data from the api
    :type json_data: dict

    :Acessible Paramters:
        :id: [:class:`int`] ID of the person 
        :name: [:class:`str`] Discord username of the person 
        :token: [:class:`str`] Fluxpoint token of the person 
        :created: [:class:`datetime.datetime`] Account creation date
        :expire: [Optional[:class:`datetime.datetime`]] Expiration date
        :donator: [:class:`bool`] If the perso is a dontor in fluxpoint or not
        :owner: [:class:`bool`] If the person is owner or not
        :disabled: [:class:`bool`] If the acc is disabled or not
        :status: [:class:`str`] HTTP Status
        :code: [:class:`int`] HTTP Status code
        :message: [:class:`str`] Message returned by the api
    """

    def __init__(self, json_data: dict) -> None:
        for i in json_data:
            setattr(self, i.lower(), json_data[i])

    def __str__(self) -> str:
        return f"<AboutUs {self.name} ({self.id})>"


class MinecraftPingData:
    """A class which gives info from the json data of Minecraft server ping

    :param json_data: The json data from the api
    :type json_data: dict

    :Acessible Paramters:
        :online: [:class:`bool`] If the server is online or not
        :icon: [:class:`str`] URL of the server icon
        "motd": [:class:`str`] serverinfo
        :playersOnline: [:class:`int`] Amount of players online
        :playersMax: [:class:`int`] Max amount of players
        :version: [:class:`str`] Minecraft version
        :fullQuery: [:class:`bool`] If the query was full or not
        :players: [Optional[:class:`list`]] List of players
        :status: [:class:`str`] HTTP Status
        :code: [:class:`int`] HTTP Status code
        :message: [:class:`str`] Message returned by the api
    """

    def __init__(self, json_data: dict) -> None:
        for i in json_data:
            setattr(self, i.lower(), json_data[i])

    def __str__(self) -> str:
        return '<Minecraft Server Ping Data>'

    def __repr__(self) -> str:
        return '<Minecraft Server Ping Data>'


class Misc(BaseHTTP):
    """NSFW Api endpoints documented in https://bluedocs.page/fluxpoint-api/misc"""

    def __str__(self) -> str:
        return '<MISC>'

    async def me(self) -> AboutUs:
        """Get info on the current user

        :return: The class containing all the information about the current user
        :rtype: AboutUs
        """
        return AboutUs(await self.request(RequestTypes.GET, 'me'))

    async def mc_ping(self, host: str, port: Optional[int] = 25565) -> MinecraftPingData:
        """Ping a minecraft server

        :param host: The hostname of the server
        :type host: str
        :param port: The port of the server
        :type port: int
        """
        return MinecraftPingData(await self.request(RequestTypes.GET, f'mc/ping?host={host}&{port}'))  # skipcq: FLK-E50



from .paths import Gifs, Images, NSFW, Misc, Welcome, ImageGenerator


class FluxpointClient(Gifs, Images, NSFW, Misc, Welcome, ImageGenerator):
    '''The fluxpoint client where all the api routes are located'''

    def __str__(self) -> str:
        return f'<Fluxpoint Client api_token={self.api_token}>'



from typing import Union

from yarl import URL

from ..enums import RequestTypes
from ..http import BaseHTTP


class Gifs(BaseHTTP):
    """GIFS Api endpoints documented in https://bluedocs.page/fluxpoint-api/gifs"""

    def __str__(self) -> str:
        return '<Gifs>'

    async def baka(self) -> Union[URL, str]:
        """
        Returns a random baka gif

        :return: Url of the image
        :rtype: Union[:class:`yarl.URL` , :class:`str`]
        """
        return (await self.request(RequestTypes.GET, 'api/sfw/gif/baka', _base_url='https://gallery.fluxpoint.dev/'))['file']

    async def bite(self) -> Union[URL, str]:
        """
        Returns a random bite gif

        :return: Url of the image
        :rtype: Union[:class:`yarl.URL` , :class:`str`]
        """
        return (await self.request(RequestTypes.GET, '/api/sfw/gif/bite', _base_url='https://gallery.fluxpoint.dev/'))['file']

    async def blush(self) -> Union[URL, str]:
        """
        Returns a random blush gif

        :return: Url of the image
        :rtype: Union[:class:`yarl.URL` , :class:`str`]
        """
        return (await self.request(RequestTypes.GET, '/api/sfw/gif/blush', _base_url='https://gallery.fluxpoint.dev/'))['file']

    async def cry(self) -> Union[URL, str]:
        """
        Returns a random cry gif

        :return: Url of the image
        :rtype: Union[:class:`yarl.URL` , :class:`str`]
        """
        return (await self.request(RequestTypes.GET, '/api/sfw/gif/cry', _base_url='https://gallery.fluxpoint.dev/'))['file']

    async def dance(self) -> Union[URL, str]:
        """
        Returns a random dance gif

        :return: Url of the image
        :rtype: Union[:class:`yarl.URL` , :class:`str`]
        """
        return (await self.request(RequestTypes.GET, '/api/sfw/gif/dance', _base_url='https://gallery.fluxpoint.dev/'))['file']

    async def feed(self) -> Union[URL, str]:
        """
        Returns a random feed gif

        :return: Url of the image
        :rtype: Union[:class:`yarl.URL` , :class:`str`]
        """
        return (await self.request(RequestTypes.GET, '/api/sfw/gif/feed', _base_url='https://gallery.fluxpoint.dev/'))['file']

    async def fluff(self) -> Union[URL, str]:
        """
        Returns a random fluff gif

        :return: Url of the image
        :rtype: Union[:class:`yarl.URL` , :class:`str`]
        """
        return (await self.request(RequestTypes.GET, '/api/sfw/gif/fluff', _base_url='https://gallery.fluxpoint.dev/'))['file']

    async def grab(self) -> Union[URL, str]:
        """
        Returns a random grab cheeks gif

        :return: Url of the image
        :rtype: Union[:class:`yarl.URL` , :class:`str`]
        """
        return (await self.request(RequestTypes.GET, '/api/sfw/gif/grab', _base_url='https://gallery.fluxpoint.dev/'))['file']

    async def grabcheecks(self) -> Union[URL, str]:
        """
        Returns a random grab cheeks gif

        :return: Url of the image
        :rtype: Union[:class:`yarl.URL` , :class:`str`]
        """
        return (await self.request(RequestTypes.GET, '/api/sfw/gif/grab', _base_url='https://gallery.fluxpoint.dev/'))['file']

    async def handhold(self) -> Union[URL, str]:
        """
        Returns a random hand holding gif

        :return: Url of the image
        :rtype: Union[:class:`yarl.URL` , :class:`str`]
        """
        return (await self.request(RequestTypes.GET, '/api/sfw/gif/handhold', _base_url='https://gallery.fluxpoint.dev/'))['file']

    async def handholding(self) -> Union[URL, str]:
        """
        Returns a random hand holding gif

        :return: Url of the image
        :rtype: Union[:class:`yarl.URL` , :class:`str`]
        """
        return (await self.request(RequestTypes.GET, '/api/sfw/gif/handhold', _base_url='https://gallery.fluxpoint.dev/'))['file']

    async def hand_holding(self) -> Union[URL, str]:
        """
        Returns a random hand holding gif

        :return: Url of the image
        :rtype: Union[:class:`yarl.URL` , :class:`str`]
        """
        return (await self.request(RequestTypes.GET, '/api/sfw/gif/handhold', _base_url='https://gallery.fluxpoint.dev/'))['file']

    async def hand_hold(self) -> Union[URL, str]:
        """
        Returns a random hand holding gif

        :return: Url of the image
        :rtype: Union[:class:`yarl.URL` , :class:`str`]
        """
        return (await self.request(RequestTypes.GET, '/api/sfw/gif/handhold', _base_url='https://gallery.fluxpoint.dev/'))['file']

    async def highfive(self) -> Union[URL, str]:
        """
        Returns a random highfive gif

        :return: Url of the image
        :rtype: Union[:class:`yarl.URL` , :class:`str`]
        """
        return (await self.request(RequestTypes.GET, '/api/sfw/gif/highfive', _base_url='https://gallery.fluxpoint.dev/'))['file']

    async def hug(self) -> Union[URL, str]:
        """
        Returns a random hug gif

        :return: Url of the image
        :rtype: Union[:class:`yarl.URL` , :class:`str`]
        """
        return (await self.request(RequestTypes.GET, '/api/sfw/gif/hug', _base_url='https://gallery.fluxpoint.dev/'))['file']

    async def kiss(self) -> Union[URL, str]:
        """
        Returns a random kiss gif

        :return: Url of the image
        :rtype: Union[:class:`yarl.URL` , :class:`str`]
        """
        return (await self.request(RequestTypes.GET, '/api/sfw/gif/kiss', _base_url='https://gallery.fluxpoint.dev/'))['file']

    async def lick(self) -> Union[URL, str]:
        """
        Returns a random lick gif

        :return: Url of the image
        :rtype: Union[:class:`yarl.URL` , :class:`str`]
        """
        return (await self.request(RequestTypes.GET, '/api/sfw/gif/lick', _base_url='https://gallery.fluxpoint.dev/'))['file']

    async def neko_gif(self) -> Union[URL, str]:
        """
        Returns a random neko gif

        :return: Url of the image
        :rtype: Union[:class:`yarl.URL` , :class:`str`]
        """
        return (await self.request(RequestTypes.GET, '/api/sfw/gif/neko', _base_url='https://gallery.fluxpoint.dev/'))['file']

    async def pat(self) -> Union[URL, str]:
        """
        Returns a random pat gif

        :return: Url of the image
        :rtype: Union[:class:`yarl.URL` , :class:`str`]
        """
        return (await self.request(RequestTypes.GET, '/api/sfw/gif/pat', _base_url='https://gallery.fluxpoint.dev/'))['file']

    async def poke(self) -> Union[URL, str]:
        """
        Returns a random poke gif

        :return: Url of the image
        :rtype: Union[:class:`yarl.URL` , :class:`str`]
        """
        return (await self.request(RequestTypes.GET, '/api/sfw/gif/poke', _base_url='https://gallery.fluxpoint.dev/'))['file']

    async def punch(self) -> Union[URL, str]:
        """
        Returns a random punch gif

        :return: Url of the image
        :rtype: Union[:class:`yarl.URL` , :class:`str`]
        """
        return (await self.request(RequestTypes.GET, '/api/sfw/gif/punch', _base_url='https://gallery.fluxpoint.dev/'))['file']

    async def shrug(self) -> Union[URL, str]:
        """
        Returns a random shrug gif

        :return: Url of the image
        :rtype: Union[:class:`yarl.URL` , :class:`str`]
        """
        return (await self.request(RequestTypes.GET, '/api/sfw/gif/shrug', _base_url='https://gallery.fluxpoint.dev/'))['file']

    async def slap(self) -> Union[URL, str]:
        """
        Returns a random slap gif

        :return: Url of the image
        :rtype: Union[:class:`yarl.URL` , :class:`str`]
        """
        return (await self.request(RequestTypes.GET, '/api/sfw/gif/slap', _base_url='https://gallery.fluxpoint.dev/'))['file']

    async def smug(self) -> Union[URL, str]:
        """
        Returns a random smug gif

        :return: Url of the image
        :rtype: Union[:class:`yarl.URL` , :class:`str`]
        """
        return (await self.request(RequestTypes.GET, '/api/sfw/gif/smug', _base_url='https://gallery.fluxpoint.dev/'))['file']

    async def stare(self) -> Union[URL, str]:
        """
        Returns a random stare gif

        :return: Url of the image
        :rtype: Union[:class:`yarl.URL` , :class:`str`]
        """
        return (await self.request(RequestTypes.GET, '/api/sfw/gif/stare', _base_url='https://gallery.fluxpoint.dev/'))['file']

    async def tickle(self) -> Union[URL, str]:
        """
        Returns a random tickle gif

        :return: Url of the image
        :rtype: Union[:class:`yarl.URL` , :class:`str`]
        """
        return (await self.request(RequestTypes.GET, '/api/sfw/gif/tickle', _base_url='https://gallery.fluxpoint.dev/'))['file']

    async def wag(self) -> Union[URL, str]:
        """
        Returns a random wag gif

        :return: Url of the image
        :rtype: Union[:class:`yarl.URL` , :class:`str`]
        """
        return (await self.request(RequestTypes.GET, '/api/sfw/gif/wag', _base_url='https://gallery.fluxpoint.dev/'))['file']

    async def wasted(self) -> Union[URL, str]:
        """
        Returns a random wasted gif

        :return: Url of the image
        :rtype: Union[:class:`yarl.URL` , :class:`str`]
        """
        return (await self.request(RequestTypes.GET, '/api/sfw/gif/wasted', _base_url='https://gallery.fluxpoint.dev/'))['file']

    async def wave(self) -> Union[URL, str]:
        """
        Returns a random wave gif

        :return: Url of the image
        :rtype: Union[:class:`yarl.URL` , :class:`str`]
        """
        return (await self.request(RequestTypes.GET, '/api/sfw/gif/wave', _base_url='https://gallery.fluxpoint.dev/'))['file']

    async def wink(self) -> Union[URL, str]:
        """
        Returns a random wink gif

        :return: Url of the image
        :rtype: Union[:class:`yarl.URL` , :class:`str`]
        """
        return (await self.request(RequestTypes.GET, '/api/sfw/gif/wink', _base_url='https://gallery.fluxpoint.dev/'))['file']



import asyncio
import io
from typing import Optional, Union

import aiohttp
from yarl import URL

from . import __version__
from .enums import RequestTypes
from .errors import *


class BaseHTTP:
    """The base class for making http requests

    :param api_token: The fluxpoint api token https://fluxpoint.dev/api/access
    :type api_token: str
    """
    __slots__ = ["api_token"]

    def __init__(self, api_token: str) -> None:
        self.api_token: str = api_token
        self.__user_agent: str = f"fluxpoint/{__version__}"

    async def request(
        self,
        method: RequestTypes,
        endpoint: str,
        json: Optional[dict] = None,
        headers: Optional[dict] = None,
        _base_url: Optional[Union[str, URL]] = 'https://api.fluxpoint.dev/',
        retry: bool = True,
        return_json: bool = True,
        return_bytes: bool = False,
        retry_times: int = 1
    ) -> Union[aiohttp.ClientResponse, dict, io.IOBase]:
        """Makes an API request

        :raises RateLimited: When the 429 response is returned
        :raises WrongReturnType: When the :exc:`UnicodeDecodeError` is raised
        :raises ParameterError: When the 400 response is returned
        :raises Unauthorised: When the 401 response is returned
        :raises ApiError: When the 500 response is returned
        :raises HttpException: For the other and generall http exceptions
        :return: Bytes data for the image
        :rtype: Union[aiohttp.ClientResponse, dict, io.IOBase]
        """
        if json is None:
            json = {}
        __base_url: str = _base_url if _base_url.endswith(
            '/') else _base_url.strip() + '/'
        headers = {} if not headers else headers

        headers["Authorization"] = self.api_token
        headers["User-Agent"] = self.__user_agent

        async with aiohttp.ClientSession() as session:
            async with session.request(str(method.name).upper(), f'{__base_url}{str(endpoint)}', headers=headers, json=json) as response:
                if response.status == 429:
                    if not retry:
                        raise RateLimited("Too many requests, try again later")
                    await asyncio.sleep(response.headers.get('Retry-After'))
                    return await self.request(method, endpoint, json, headers, retry=retry_times <= 10, retry_times=retry_times+1)

                try:
                    result = await response.json(content_type="application/json") if return_json else (await response.read()if return_bytes else response)
                    if response.status == 200:
                        return result
                    result = await response.json(content_type="application/json") if not isinstance(result, dict) else result
                except UnicodeDecodeError:
                    raise WrongReturnType("Wrong return type is given")

                if response.status == 400:
                    raise ParameterError(result.get("message"))

                if response.status == 401:
                    raise Unauthorised(result.get("message"))

                if response.status == 500:
                    raise ApiError(result.get("message"))

                raise HttpException(response.status, result)



from setuptools import setup
import re

pakage_name = 'fluxpoint'
requirements = []
with open('requirements.txt') as f:
    requirements = f.read().splitlines()

version = ''
with open(f'{pakage_name}/__init__.py') as f:
    version = re.search(
        r'^__version__\s*=\s*[\'"]([^\'"]*)[\'"]', f.read(), re.MULTILINE).group(1)

if not version:
    raise RuntimeError('version is not set')

if version.endswith(('a', 'b', 'rc')):
    # append version identifier based on commit count
    try:
        import subprocess
        p = subprocess.Popen(['git', 'rev-list', '--count', 'HEAD'], #skipcq : BAN-B607
                             stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        out, err = p.communicate()
        if out:
            version += out.decode('utf-8').strip()
        p = subprocess.Popen(['git', 'rev-parse', '--short', 'HEAD'],
                             stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        out, err = p.communicate()
        if out:
            version += '+g' + out.decode('utf-8').strip()
    except Exception:
        pass

readme = ''
with open('README.rst') as f:
    readme = f.read()

extras_require = {
    'speed': [
        "aiohttp[speedups]>=3.8.1"
    ],
    'docs': [
        'sphinx',
        'sphinxcontrib_trio',
        'sphinxcontrib-websupport',
        'sphinx-autoapi',
        'typing-extensions'
    ]

}

packages = [
    pakage_name,
    f'{pakage_name}.paths'
]

setup(
    name='fluxpoint.py',
    author='Dhruva Shaw',
    url='https://github.com/Dhruvacube/fluxpoint.py',
    project_urls={
        "fluxpoint.py API wrapper Docs": "https://fluxpointpy.readthedocs.io",
        "Official Api Docs": "https://bluedocs.page/fluxpoint-api",
        "Get Api Token": "https://fluxpoint.dev/api/access",
        "Offcial Fluxpoint discord server": "https://discord.gg/fluxpoint",
        "Support server for api wrapper": "https://discord.gg/vfXHwS3nmQ",

        "Repository": "https://github.com/Dhruvacube/fluxpoint.py",
        "Examples Directory": "https://github.com/Dhruvacube/fluxpoint.py/tree/master/examples",
        "Issue tracker": "https://github.com/Dhruvacube/fluxpoint.py/issues",
    },
    version=version,
    packages=packages,
    license='GNU GENERAL PUBLIC LICENSE',
    description='A Python wrapper for the Fluxpoint API',
    keywords="fluxpoint api, api, wrapper, async wrapper",
    long_description=readme,
    long_description_content_type="text/x-rst",
    include_package_data=True,
    install_requires=requirements,
    extras_require=extras_require,
    package_data={"fluxpoint": ["py.typed"]},
    python_requires='>=3.8',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'License :: OSI Approved :: GNU General Public License v2 (GPLv2)',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        "Programming Language :: Python :: 3",
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Topic :: Internet',
        'Topic :: Communications',
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Utilities',
        'Typing :: Typed',
    ]
)
